* Michael Tomz and Robert Van Houweling, "Candidate Positioning and Voter Choice,"
* American Political Science Review 102, no. 3 (August 2008): 303-318.
*
* Purpose: This do-file analyzes data
* Version: February 21, 2010
* Syntax:  Stata 9.2
version 9.2
set more off
capture log close
log using output, text replace
quietly log off
local makefigs yes   // yes = make figures (takes more time), no = do not make figures (runs faster)

*************************************************
* DEFINE SCENARIOS AND BUILD DEPENDENT VARIABLE *
*************************************************

   use TomzVanHouweling_APSR_2008, clear    /* load data */
   keep if qflag == 1                       /* keep qualifiers */
   
   ********************
   * DEFINE SCENARIOS *
   ********************
   gen cbar = (candleft + candright)/2      /* midpoint between the two candidates */
   gen astar = (self - cbar) / (sq - cbar)  /* alpha-star, the cutoff value for discounting */
   gen byte scennum = .  /* initialize scenario number */
   replace scennum = 1 if 0 < self & self < cbar & self <= sq    /* Scenario I: N < v < cbar, v <= Q */
   replace scennum = 1 if cbar < self & self < 0 & sq <= self    /* Reflection of scenario I */
   replace scennum = 2 if self < cbar & self < 0 & self <= sq    /* Scenario II: v < cbar,N and v <= Q */
   replace scennum = 2 if cbar < self & 0 < self & sq <= self    /* Reflection of scenario II */
   replace scennum = 3 if sq < self & self < cbar & self < 0     /* Scenarios III, IV: Q < v < cbar, N */
   replace scennum = 3 if cbar < self & 0 < self & self < sq     /* Reflection of Scenario III,IV */
   replace scennum = 4 if 0 < self & sq < self & self < cbar     /* Senarios V, VI: N,Q < v < cbar */
   replace scennum = 4 if cbar < self & self < 0 & self < sq     /* Reflection of scenarios V, VI */
         
		 
   *********************************************************************************************
   * BUILD THE DEPENDENT VARIABLE: DID RESPONDENT CHOSE "C_2" (As defined in Table 1, p. 308)? *
   *********************************************************************************************
   gen spleft = self * candleft    /* scalar product of self and left candidate */
   gen spright = self * candright  /* scalar product of self and right candidate */
   gen byte pickdf = cond((pickleft==1 & (spleft>spright))|(pickleft==0 & (spleft<spright)),1,0) /* picked dir favorite */
   gen byte pickc2 = 0                                 /* initialize to be zero */
   replace pickc2 = 1 if pickdf == 1 & scennum==1      /* picked c2 in scenario I */
   replace pickc2 = 1 if pickdf == 0 & scennum==2      /* picked c2 in scenario II */
   replace pickc2 = 1 if pickdf == 0 & scennum==3      /* picked c2 in scenario III, IV */
   replace pickc2 = 1 if pickdf == 1 & scennum==4      /* picked c2 in scenario V, VI */
   

   **************************
   * DESCRIPTIVE STATISTICS *
   **************************
   quietly log on 
   * Table 2, p. 312
   tab scennum pickc2 if scennum==1 | scennum==2 | (scennum>2 & astar < .1), row  
   * Footnote 16
   gen extremist = cond(candleft==-5 | candright==5,1,0)  /* At least 1 candidate at the pole */
   tab scennum pickc2 if (scennum==1 | scennum==2 | (scennum>2 & astar < .1)) & extremist==0, row
   * Footnote 17 (uninformative subsets of Scenarios III, IV and of Scenarios V, VI)
   tab scennum pickc2 if scennum>2 & astar >= .1, row 
   quietly log off

   
   *************************
   * KEEP INFORMATIVE DATA *
   *************************
   keep if scennum==1 | scennum==2 | (scennum>2 & astar < .1) 
   describe, short   /* N = 1,045 informative cases (see p. 312) */
   drop pickdf astar dt_start
   tempfile data
   save `data'


*******************
* POOLED ANALYSIS *
*******************

   ***********************************************
   * ESTIMATED RATES OF PROX, DISC, DIR (p. 312) *
   ***********************************************

   quietly log on
   di _n(2) "Main estimates"
   est_rates
   return list
   quietly log off

   if "`makefigs'" == "yes" {
	   
	   ***************************************************************************************
	   * FIGURE 4: point estimates and bootsrapped confidence region via convex hull peeling *
	   ***************************************************************************************
	   * note: to replicate footnote 19 by constructing confidence regions via 
	   * Mahalanobis Distance, change method(chp) to method(mah) in the commands below
	   set seed 2468
	   local nboots 10000  /* number of bootstraps */
	   est_rates_boot, nboots(`nboots') method(chp) saveboots("saveboots")  /* NAC: ADDED SAVEBOOTS */

	   * these are coordinates for guide lines (rays) to the
	   * point estimates in the ternary graph
	   local y1 = `r(piprox)' *sqrt(3)/2           // v1 * height 
	   local x1 = `r(piprox)'/2 + `r(pidisc)'      // v1/2 + v2
	   local y2 = `y1'                             // v1 * height
	   local x2 = `r(piprox)'/2                    // v1/2
	   local y3 = (1 - `r(pidisc)' ) * sqrt(3)/2   // (1-v2)*height
	   local x3 = .5 + `r(pidisc)'/2               // .5 + v2/2
	   local y4 = 0                                // zero
	   local x4 = `r(piprox)' + `r(pidisc)'        // v1 + v2
	   local y4prime = -1/33
	   local x4prime = `x4' - .01

	   * make ternary plot
	   triplotmt piprox pidisc pidir ,  /*
		  */ y y(clpattern(longdash)) max(100) label(20 40 60 80) /*
		  */ grid(connect(i)) msymbol(circle) ltext(" ") rtext(" ") /*
		  */ btext(" ") bltext(Directional) brtext(Discounting) /*
		  */ ttext(Proximity) scheme(s1mono) /*
		  */ moregraphs( /*
		  */ text(`y2' `x2' "57.7%   ", placement(w) size(small)) /*
		  */ text(`y3' `x3' "  27.6%", placement(ne) size(small)) /*
		  */ text(`y4prime' `x4prime' "14.7%", placement(e) size(small)) /*
		  */ || pci `y1' `x1' `y2' `x2', lcolor(black) lpattern(dot) /*
		  */ || pci `y1' `x1' `y4' `x4', lcolor(black) lpattern(dot) /*
		  */ || pci `y1' `x1' `y3' `x3', lcolor(black) lpattern(dot) /*
		  */ || line y_boot x_boot, lcolor(black) sort(hullorder) ) 

	   graph export figure4.eps, replace logo(off) mag(150) 
	   
   }

  
*********************************
* ANALYSIS BY DEMOGRAPHIC GROUP *
*********************************

   *********************************************************
   * ESTIMATES BY EDUCATION, PARTY, IDEOLOGY (pp. 313-314) *
   *********************************************************

   use `data', clear
   quietly log on
   di _n(2) "Estimates by educational group (with college degree vs no college degree)"
   tab ed4
   est_rates_bygroup, group1(ed4 == 4) group2(ed4 < 4)
   return list
   di _n(2) "Estimates by strength of party ID (strong partisans vs independents & leaners)"
   tab pid7
   est_rates_bygroup, group1(pid7 == 1 | pid7 == 7) group2(pid7 == 3 | pid7 == 4 | pid7 == 5)
   return list
   di _n(2) "Estimates by strength of ideology (strong vs. neutral ideology)"
   tab ideo7
   est_rates_bygroup, group1(ideo7 == 1 | ideo7 == 2 | ideo7 == 6 | ideo7 == 7) group2(ideo7 == 4)
   return list
   quietly log off

   if "`makefigs'" == "yes" {
   
	   *********************
	   * FIGURE 5 (p. 314) *
	   *********************
	 
	   tempfile g1 g2 g3 firstpart
	   set seed 2468

	   * COLLEGE DEGREE
	   use `data', clear
	   keep if ed4 == 4  /* college degree */
	   est_rates_boot, nboots(`nboots') method(chp)
	   gen str1 mletter = "H"
	   save `firstpart', replace

	   * NO COLLEGE DEGREE
	   use `data', clear
	   keep if ed4 < 4  /* Less than college degree */
	   est_rates_boot, nboots(`nboots') method(chp)
	   gen str1 mletter = "L"
	   append using `firstpart'

	   * PLOT OF COLLEGE DEGREE VERSUS NO COLLEGE DEGREE
	   sort mletter hullorder
	   local vtext ttext(Proximity) brtext(Discounting) bltext(Directional)  /* text at vertices */
	   triplotmt piprox pidisc pidir , max(100) /*
		  */ aspectratio(.775) title("(a) High vs. Low Education") /*
		  */ mlabel(mletter) mlabposition(0) symbol(i) y(clpattern(longdash)) /*
		  */ y label(nolabels) msymbol(circle) ltext(" ") rtext(" ") btext(" ") /*
		  */ `vtext' scheme(s1mono) moregraphs( /*
		  */ || line y_boot x_boot if mletter=="H", lcolor(black) sort(hullorder) /*
		  */ || line y_boot x_boot if mletter=="L", lcolor(black) sort(hullorder) lpattern(dot) )
	   graph save `g1'

	   * STRONG PARTISAN
	   use `data', clear
	   keep if pid7 == 1 | pid7 == 7 /* strong identifier */
	   est_rates_boot, nboots(`nboots') method(chp)
	   gen str1 mletter = "P"
	   save `firstpart', replace

	   * INDEPENDENT OR LEANER
	   use `data', clear
	   keep if pid7 == 3 | pid7 == 4 | pid7 == 5
	   est_rates_boot, nboots(`nboots') method(chp)
	   gen str1 mletter = "I"
	   append using `firstpart'

	   * PLOT OF STRONG PARTISAN VERSUS INDEPENDENT OR LEANER
	   local vtext ttext(Proximity) brtext(Discounting) bltext(Directional)  /* text at vertices */
	   triplotmt piprox pidisc pidir , max(100) /*
		  */ aspectratio(.775) title("(b) Partisans vs. Independents") /*
		  */ mlabel(mletter) mlabposition(0) symbol(i) y(clpattern(longdash)) /*
		  */ y label(nolabels) msymbol(circle) ltext(" ") rtext(" ") btext(" ") /*
		  */ `vtext' scheme(s1mono) moregraphs( /*
		  */ || line y_boot x_boot if mletter=="P", lcolor(black) sort(hullorder) /*
		  */ || line y_boot x_boot if mletter=="I", lcolor(black) sort(hullorder) lpattern(dot) )
	   graph save `g2'

	   * STRONG IDEOLOGUE
	   use `data', clear
	   keep if ideo7 == 1 | ideo7 == 2 | ideo7 == 6 | ideo7 == 7 /* strong ideologue */
	   est_rates_boot, nboots(`nboots') method(chp)
	   gen str1 mletter = "S"
	   save `firstpart', replace

	   * NON-IDEOLOGUE
	   use `data', clear
	   keep if ideo7 == 4  /* neutral */
	   est_rates_boot, nboots(`nboots') method(chp)
	   gen str1 mletter = "N"
	   append using `firstpart'

	   * PLOT OF STRONG IDOELOGUE VERSUS NON-IDEOLOGUE
	   local vtext ttext(Proximity) brtext(Discounting) bltext(Directional)  /* text at vertices */
	   triplotmt piprox pidisc pidir , max(100) /*
		  */ aspectratio(.775) title("(c) Strong vs. Neutral Ideology") /*
		  */ mlabel(mletter) mlabposition(0) symbol(i) y(clpattern(longdash)) /*
		  */ y label(nolabels) msymbol(circle) ltext(" ") rtext(" ") btext(" ") /*
		  */ `vtext' scheme(s1mono) moregraphs( /*
		  */ || line y_boot x_boot if mletter=="N", lcolor(black) sort(hullorder) lpattern(dot) /*
		  */ || line y_boot x_boot if mletter=="S", lcolor(black) sort(hullorder) )
	   graph save `g3'

	   * COMBINE THE THREE PLOTS
	   graph combine "`g1'" "`g2'" "`g3'", rows(3) ysize(5.5) xsize(4.25) scheme(s1mono)
	   graph export figure5.eps, replace logo(off) mag(200)

   }
   

***************************************************
* APPENDIX B: ALTERNATIVE ASSUMPTIONS ABOUT ERROR *
***************************************************

   ****************
   * Prepare data *
   ****************
   use `data', clear
   gen d_cbar = abs(self-cbar)              /* voter's distance to cbar */
   gen d_n = abs(self)                      /* voter's absolute distance to neutral point */
   gen d_btwn = abs(candleft - candright)   /* absolute distance between candidates */
   gen d_sp = abs(spleft - spright)/5       /* absolute diff in scalar products, normalized by 5 */
   global S scennum
   global dcbar d_cbar
   global dn d_n
   global dbtwn d_btwn
   global dsp d_sp
   tempfile data2
   save `data2'

   
   ****************************************************************
   * ASSUME ALL VOTERS ERR AT THE SAME RATE (BENCHMARK ESTIMATES) *
   ****************************************************************
   quietly log on
   di _n(2) "****** Benchmark estimates via maximum likelihood ******" _n(1)
   ml model lf errequal (et: pickc2=) (pdirt:) (pdisct:)
   quietly ml search
   quietly ml max
   errequal_results 
  
  
   ************************************************
   * ASSUME A HIGHER ERROR RATE FOR "CLOSE CALLS" *
   ************************************************
  
   foreach a in .15 .25 .35 .45 {   // loop through presumed values of alpha

      quietly use `data2', clear
      capture drop pbar d_pbar
      gen pbar = `a'*sq + (1-`a')*cbar  /* assume `a' weight to the status quo */
      gen d_pbar = abs(self-pbar)       /* voter's distance to pbar */
      global dpbar d_pbar

      di in r _n(3) "****** Individual-specific error: Directional |v-N|, Discounting alpha = `a' ********" _n(1)
      ml model lf errequal_dn (et: pickc2=) (pdirt:) (pdisct:) (ff:)
      quietly ml search
      quietly ml max
	  errequal_results
	  di in y "g    : " %6.3f _b[ff:_cons]
	  
      di in r _n(3) "****** Individual-specific error: Directional |c1-c2|, Discounting alpha = `a' ********" _n(1)
      ml model lf errequal_dbtwn (et: pickc2=) (pdirt:) (pdisct:) (ff:)
      quietly ml search
      quietly ml max
      errequal_results
	  di in y "g    : " %6.3f _b[ff:_cons]
	  
      di in r _n(3) "****** Individual-specific error: Directional |v*c1 - v*c2|/5, Discounting alpha = `a' ********" _n(1)
      ml model lf errequal_dsp (et: pickc2=) (pdirt:) (pdisct:) (ff:)
      quietly ml search
      quietly ml max
      errequal_results
      di in y "g    : " %6.3f _b[ff:_cons]
	  
   }
   quietly log off
      
   ***************************
   * "Random choosers" model *
   ***************************

   su pickc2 if scennum==1, meanonly
   local pi_I = `r(mean)'
   su pickc2 if scennum==2, meanonly
   local pi_II = `r(mean)'
   su pickc2 if scennum==4, meanonly
   local pi_VIstar = `r(mean)'
   local pi_dir = `pi_I' - `pi_II'
   local pi_disc = `pi_VIstar' - `pi_dir' - `pi_II'
   quietly log on
   di in r _n(3) "****** Model with random choosers ******"
   di "Proximity  : " %5.4f 1 - `pi_dir' - `pi_disc' - 2*`pi_II'
   di "Discounting: " %5.4f `pi_disc'
   di "Directional: " %5.4f `pi_dir'
   di "Pure random: " %5.4f 2*`pi_II'
   quietly log off

***********************
* ADDITIONAL ANALYSIS *
***********************

   * Compare effciency of our algorithm for locating candidates, versus two alternative algrorithms:
   * complete randomization or randomization designed to match the ANES (p. 310 and fn 11)
   quietly log on
   footnote11  // this takes a while due to large number of simulations
   log close

exit
 
