program define est_rates_boot, rclass
  * Michael Tomz and Robert Van Houweling, "Candidate Positioning and Voter Choice,"
  * American Political Science Review 102, no. 3 (August 2008): 303-318.
  *
  * Purpose: Estimates rates of prox, disc, dir, and common error rate
  *    gives point estimates, bootstrapped se's, and confidence region (if requested)
  *    the confidence region can be calcuated in two ways
  *       method(chp) uses Convex Hull Peeling [this is the default]
  *       method(mah) uses Mahalanobis depth 
  *    user can specify confidence level and # of bootstraps [default=1000]
  *    user can specify whether to save files [saveboots(str)]
  *
  * Version: February 21, 2010
  * Syntax:  Stata 9.2
  version 9.2
  syntax [if] [in], [method(str) level(int 50) nboots(int 1000) saveboots(str)]

  * if user wants to save the bootstrap estimates or generate 
  * a confidence region for the ternary plot ....
  if "`method'" ~= "" | "`saveboots'" ~= "" {
	 tempfile boots  /* bootstrap estimates stored in a temporary dataset called boots */
	 bootstrap piprox_boot=r(piprox) pidisc_boot=r(pidisc) pidir_boot=r(pidir) /* 
		*/ err=r(error) `if' `in', /*
		*/ reps(`nboots') saving(`boots', double replace) nowarn : est_rates
	 use `boots', clear
	 * Recode negative estimates to be zero
	 di in y _n(1) "Recoding ..."
	 replace pidir_boot = 0 if pidir_boot < 0  
	 replace pidisc_boot = 0 if pidisc_boot < 0  
	 replace piprox_boot = 1 - pidisc_boot - pidir_boot
	 di _n
	 * save the file if requested
	 if "`saveboots'" ~= "" {
		save `saveboots'
	 }
  }
  * if user does not want to save the bootsrap estimates or
  * generate a confidence region for the ternary plot ....
  else {
	 bootstrap piprox_boot=r(piprox) pidisc_boot=r(pidisc) pidir_boot=r(pidir) /* 
		*/ err=r(error) `if' `in', reps(`nboots') nowarn : est_rates
  }

  * save maximum likelihood estimates for proximity, 
  * discounting, directional voting, and error
  if "`method'" == "" {
	 return local piprox = _b[piprox_boot]
	 return local pidisc = _b[pidisc_boot]
	 return local pidir = _b[pidir_boot]
	 return local err = _b[err]
	 exit /* terminate the procedure; don't execute subsequent lines */
  }

  * code below this point is executed only if user has
  * specified a method for ternary confidence region
  local piprox = _b[piprox_boot]
  local pidisc = _b[pidisc_boot]
  local pidir = _b[pidir_boot]
  local err = _b[err]

  * x, y coordinates for bootstrapped points
  gen y_boot = (sqrt(3)/2) * piprox_boot
  gen x_boot = pidisc_boot + piprox_boot / 2
  drop pidir_boot pidisc_boot piprox_boot
  tempvar inhull

  * mah method
  if "`method'" == "mah" {
	 local y_mle = (sqrt(3)/2) * `piprox'    /* project MLE onto ternary plot */
	 local x_mle = `pidisc' + `piprox' / 2   /* project MLE onto ternary plot */
	 mdist y_boot x_boot, mean1(`y_mle') mean2(`x_mle') generate(mdist)
	 sort mdist
	 local lowdist_row = `nboots'*`level'/100
	 local highdist_row = `lowdist_row' + 1
	 scatter y_boot x_boot in 1/`lowdist_row' || scatter y_boot x_boot in `highdist_row'/`nboots'
	 keep in 1/`lowdist_row'  /* keep only the `level'% interior points */
	 grahamscan y_boot x_boot, generate(`inhull')  /* find convex hull of remaining points */
	 * scatter y_boot x_boot if `inhull' == 0 || scatter y_boot x_boot if `inhull'==1 || line y_boot x_boot if `inhull'==1
	 quietly keep if `inhull' == 1
	 gen hullorder = _n
  }

  * chp method
  if "`method'" == "chp" {      
	 tempvar `inhull'
	 local npoints _N                           /* initialize number of points */
	 local dpoints = `npoints'*`level'/100      /* desired number of points in hull */
	 grahamscan y_boot x_boot, generate(`inhull') /* find outer hull */
	 while `npoints' > `dpoints' {
		quietly drop if `inhull' == 1           /* peel away previous hull */
		drop `inhull'
		local npoints _N                        /* number of points remaining */
		di "Hull has " _N " points (" 100*(_N)/`nboots' "% of bootstrap sample)"
		grahamscan y_boot x_boot, generate(`inhull')  /* find outer hull */
	 }
	 * scatter y_boot x_boot if `inhull' == 0 || scatter y_boot x_boot if `inhull'==1 || line y_boot x_boot if `inhull'==1
	 quietly keep if `inhull' == 1
	 gen hullorder = _n
  }

  * generate variables containing point estimates
  gen piprox = `piprox'*100
  gen pidisc = `pidisc'*100
  gen pidir = `pidir'*100

  return local piprox `piprox'
  return local pidisc `pidisc'
  return local pidir `pidir'
  return local err `err'
end


program define mdist
  version 9.2
  * this program calculates Mahalanobis for two dimensions
  * inputs are the two variables, their means, and name for new distance variable
  syntax varlist (min=2 max=2 numeric) , [mean1(real 0) mean2(real 0) generate(str)]
  tokenize `varlist'
  args x1 x2
  tempvar x1n x2n
  gen `x1n' = `x1' - `mean1'
  gen `x2n' = `x2' - `mean2'
  corr `x1' `x2', cov
  mat SigInv = inv( (`r(Var_1)', `r(cov_12)' \ `r(cov_12)', `r(Var_2)') )
  gen `generate' = `x1n'*(`x1n'*SigInv[1,1] + `x2n'*SigInv[2,1]) + /*
	 */ `x2n'*(`x1n'*SigInv[2,1] + `x2n'*SigInv[2,2])  /* Mahalanobis distance */
end


program define counterclock, rclass
  * inputs = y x, and 3 row numbers   
  * input the row nummbers of 3 points, and the vars that contain their coordinates
  * return is positive if the points are in counterclockwise order
  * return is zero if the points are collinear
  * return is negative if points are in clockwise order
  version 9.2
  syntax varlist(min=2 max=2), rownumbers(numlist min=3 max=3 integer)
  tokenize `varlist'
  args y x
  tokenize `rownumbers'
  args row1 row2 row3
  return local crossprod = (`x'[`row2'] - `x'[`row1'])*(`y'[`row3'] - `y'[`row1']) /*
	 */ - (`x'[`row3'] - `x'[`row1'])*(`y'[`row2'] - `y'[`row1'])
end


program define grahamscan
  * Finds convex hull via Graham's Scan Algorithm
  version 9.2
  syntax varlist (min=2 max=2), generate(str)
  tokenize `varlist'
  args y x
  * find the lower right point, p0
  gsort `y' -`x'   /* first point is lower right */
  * sort points angularly about p0
  * for ties, discard the closer points
  tempvar yt xt len angle flag_inhull
  gen `yt' = `y' - `y'[1]
  gen `xt' = `x' - `x'[1]
  gen double `len' = sqrt(`yt'^2 + `xt'^2)
  quietly gen double `angle' = acos(`xt'/`len')*180/_pi
  qui replace `angle' = 0 in 1
  gsort `angle' -`len'
  quietly duplicates drop `angle', force  /* discards closer points */
  * identify points in hull
  local inhull 2 1  /* initialize a list of rows corresponding to points in the hull */
  local rtest 3     /* test points 3,4,...,N progressively to see if they are in hull */
  while `rtest' <= _N {
	 tokenize `inhull'
	 args rnew rold                                       /* last 2 points added to hull */
	 counterclock `yt' `xt', rownumbers(`rold' `rnew' `rtest')  /* is next point to the left? */
	 if `r(crossprod)' > 0 {            /* if we have made a left-hand turn */
		local inhull `rtest' `inhull'   /* add point to the hull */
		local rtest = `rtest' + 1       /* and advance to the next candidate point */
	 }
	 else {                             /* if we have not made a left-hand turn */
		gettoken junk inhull : inhull   /* remove previous point from the hull */
	 }
  }
  gen byte `flag_inhull' = 0            /* create a flag for points that are in the hull */
  while "`inhull'" ~= "" {
	 gettoken row inhull : inhull
	 qui replace `flag_inhull' = 1 in `row'
  }
  quietly expand 2 in 1                 /* copy first observation to close the hull */
  generate `generate' = `flag_inhull'   /* write-out the flag to the dataset */
end
