program define footnote11
    * Michael Tomz and Robert Van Houweling, "Candidate Positioning and Voter Choice,"
    * American Political Science Review 102, no. 3 (August 2008): 303-318.
    *
    * Purpose: performs analysis in footnote 11 (p. 310)
    * Version: February 21, 2010
    * Syntax:  Stata 9.2   
    version 9.2

	********************************
	* Prepare data for simulations *
	********************************
	quietly {
		use TomzVanHouweling_APSR_2008, clear
		keep if qflag == 1 | self==-5 | self==0 | self==5  /* Our sample, plus R's at -5,0,+5 */
		gen indeterminate = .  /* v==0 or v==cbar */
		gen ties = .           /* v==-5, v==+5, or scennum==2 */
		gen uninformative = .  /* (scennum==3 | scennum==4) & astar >= .1 */
		gen critical = .       /* scennum==1 | ((scennum==3 | scennum==4) & astar < .1) */
		keep self sq indeterminate ties uninformative critical
		tempfile data
		save `data'
		local N = _N
	}

	*********************************************************************
	* WHAT IF WE HAD ASSIGNED CANDIDATE LOCATIONS COMPLETELY AT RANDOM? *
	*********************************************************************
	quietly {
	   set seed 2468
	   forvalues i = 1/1000 {
	 
		  **************************************************************************************
		  * randomly assign locations of candidates; don't let candidates occupy the same spot *
		  **************************************************************************************
		  gen c_a = ceil(uniform()*11)-6  /* candidate A */
		  gen c_b = ceil(uniform()*11)-6  /* candidate B */
		  local ok 0
		  while `ok' == 0 {
			 count if c_a == c_b
			 if `r(N)' == 0 {
				local ok 1
			 }
			 else {
				replace c_b = ceil(uniform()*11)-6 if c_a == c_b
			 }
		  }
		  gen candleft = min(c_a, c_b)
		  gen candright = max(c_a, c_b)

		  ********************
		  * define scenarios *
		  ********************
		  gen cbar = (candleft + candright)/2  /* midpoint between the two candidates */
		  gen astar = (self - cbar) / (sq - cbar)  /* alpha-star, the cutoff value for discounting */
		  gen byte scennum = .
		  * Note: This configuration is called "scenario I" in the published paper
		  replace scennum = 1 if 0 < self & self < cbar & self <= sq /* isolates directional for all alpha */
		  replace scennum = 1 if cbar < self & self < 0 & sq <= self /* reflection */
		  * Note: This configuration is called "scenario II" in the published paper
		  replace scennum = 2 if self < cbar & self < 0 & self <= sq /* allows calculation of pure error */
		  replace scennum = 2 if cbar < self & 0 < self & sq <= self /* reflection */
		  * Note: This configuration is called "scenario III and IV" in the published paper
		  replace scennum = 3 if sq < self & self < cbar & self < 0  /* isolates discounting if alpha sufficiently high */
		  replace scennum = 3 if cbar < self & 0 < self & self < sq  /* reflection */
		  * Note: This configuration is called "scenarios V and VI" in the published paper
		  replace scennum = 4 if 0 < self & sq < self & self < cbar /* isolates proximity if alpha sufficiently high */
		  replace scennum = 4 if cbar < self & self < 0 & self < sq
		  replace scennum = -2 if self==cbar  /* proximity theory is indeterminate */
		  replace scennum = -1 if self==0 | self==-5 | self==5  /* all three make same prediction */

		  ***************************************************************************************
		  * Record number of indeterminate cases, ties, uninformative cases, and critical tests *
		  ***************************************************************************************
		  count if self==0 | self==cbar  /* at least one theory is indeterminate */
		  replace indeterminate = `r(N)' in `i' 
		  count if self==-5 | self==5 | scennum==2  /* all three theories make same prediction */
		  replace ties = `r(N)' in `i'
		  count if (scennum==3 | scennum==4) & astar >= .1  /* uninformative */
		  replace uninformative = `r(N)' in `i'
		  count if scennum==1 | ((scennum==3 | scennum==4) & astar < .1) 
		  replace critical = `r(N)' in `i'
		  keep self sq indeterminate ties uninformative critical

	   } // end of 1000 simulations

	   ******************************************************************************************
	   * Calculate proportion of indeterminate cases, ties, uninformative cases, critical tests *
	   ******************************************************************************************
	   su critical, meanonly
	   local n_critical_sim = `r(mean)'
	   replace indeterminate = indeterminate / _N
	   replace ties = ties / _N 
	   replace uninformative = uninformative / _N
	   replace critical = critical / _N
	   su critical, meanonly
	   local p_critical_sim = `r(mean)'  /* average proportion of critical tests in simulated data */
	   local p_critical_expt = (261+320)/1564  /* informative cases in our experiment */

	}  /* end of quietly */

	*******************
	* Display results *
	*******************

	di in red _n(2) "*** Simulation in which candidates are assigned completely at random ***"
	di _n(2) "Proportion of indeterminate cases, ties, uninformative cases, and critical tests"
	su indeterminate ties uninformative critical

	di _n(2) "Compare the rate of informative tests in our sample versus simulated data"
	di "   Proportion of informative cases in our experiment: " `p_critical_expt'
	di "   Proportion of informative cases in simulated data: " `p_critical_sim'
	di "   Informative cases were " `p_critical_expt'/`p_critical_sim' " times more common in our experiment."

	di _n(2) "How large of a sample would we have needed to get an equal number of critical tests?"
	di "   Total number of informative cases in our sample    : 261+320 = 581"
	di "   Average number of informative cases in simulations : " `n_critical_sim'
	di "   # of interviewees needed to get 581 critical tests : " _N * 581 / `n_critical_sim'

		
		
	*************************************************************************************
	* WHAT IF WE HAD ASSIGNED CANDIDATE LOCATIONS TO MATCH THE DISTRIBUTION IN THE NES? *
	*************************************************************************************

	quietly {
	   use `data', clear
	   * do 1000 simulations
	   set seed 2468
	   forvalues i = 1/1000 {
	 
		  ****************************************************************************************
		  * randomly assign candidates to locations that mirror the distribution of presidential *
		  * candidates in the NES (requires mapping 7pt scale onto an 11pt scale)                *
		  ****************************************************************************************
		  use anes.dta , clear
		  drop if  VCF9085==0| VCF9085==8| VCF9085==9 | VCF9093==0| VCF9093==8| VCF9093==9
		  replace VCF9085 = VCF9085 - 4   /* put on scale of -3 to +3 */
		  replace VCF9093 = VCF9093 - 4
		  gen dem = round(((VCF9085+(uniform()-.5))*(11/7)))   /* map dem distribution onto 11-pt scale */
		  gen rep = round(((VCF9093+(uniform()-.5))*(11/7)))   /* map rep distribution onto 11-pt scale */
		  gen u = uniform()
		  sort u
		  keep in 1/`N'
		  gen candleft = min(dem,rep)
		  gen candright = max(dem,rep)
		  keep candleft candright
		  merge using `data'

		  ********************
		  * define scenarios *
		  ********************
		  gen cbar = (candleft + candright)/2  /* midpoint between the two candidates */
		  gen astar = (self - cbar) / (sq - cbar)  /* alpha-star, the cutoff value for discounting */
		  gen byte scennum = .
		  * Note: This configuration is called "scenario I" in the published paper
		  replace scennum = 1 if 0 < self & self < cbar & self <= sq /* isolates directional for all alpha */
		  replace scennum = 1 if cbar < self & self < 0 & sq <= self /* reflection */
		  * Note: This configuration is called "scenario II" in the published paper
		  replace scennum = 2 if self < cbar & self < 0 & self <= sq /* allows calculation of pure error */
		  replace scennum = 2 if cbar < self & 0 < self & sq <= self /* reflection */
		  * Note: This configuration is called "scenario III and IV" in the published paper
		  replace scennum = 3 if sq < self & self < cbar & self < 0  /* isolates discounting if alpha sufficiently high */
		  replace scennum = 3 if cbar < self & 0 < self & self < sq  /* reflection */
		  * Note: This configuration is called "scenarios V and VI" in the published paper
		  replace scennum = 4 if 0 < self & sq < self & self < cbar /* isolates proximity if alpha sufficiently high */
		  replace scennum = 4 if cbar < self & self < 0 & self < sq
		  replace scennum = -2 if self==cbar  /* proximity theory is indeterminate */
		  replace scennum = -1 if self==0 | self==-5 | self==5  /* all three make same prediction */

		  ***************************************************************************************
		  * Record number of indeterminate cases, ties, uninformative cases, and critical tests *
		  ***************************************************************************************
		  count if self==0 | self==cbar  /* at least one theory is indeterminate */
		  replace indeterminate = `r(N)' in `i' 
		  count if self==-5 | self==5 | scennum==2  /* all three theories make same prediction */
		  replace ties = `r(N)' in `i'
		  count if (scennum==3 | scennum==4) & astar >= .1  /* uninformative */
		  replace uninformative = `r(N)' in `i'
		  count if scennum==1 | ((scennum==3 | scennum==4) & astar < .1) 
		  replace critical = `r(N)' in `i'
		  keep self sq indeterminate ties uninformative critical
		  save `data', replace

	   } // end of 1000 simulations

	   ******************************************************************************************
	   * Calculate proportion of indeterminate cases, ties, uninformative cases, critical tests *
	   ******************************************************************************************
	   su critical, meanonly
	   local n_critical_sim = `r(mean)'
	   replace indeterminate = indeterminate / _N
	   replace ties = ties / _N 
	   replace uninformative = uninformative / _N
	   replace critical = critical / _N
	   su critical, meanonly
	   local p_critical_sim = `r(mean)'  /* average proportion of critical tests in simulated data */
	   local p_critical_expt = (261+320)/1564  /* informative cases in our experiment */

	}  /* end of quietly */


	*******************
	* Display results *
	*******************
	di in red _n(2) "*** Simulation in which candidates take positions that reflect NES distribution ***"
	di _n(2) "Proportion of indeterminate cases, ties, uninformative cases, and critical tests"
	su indeterminate ties uninformative critical

	di _n(2) "Compare the rate of informative tests in our sample versus simulated data"
	di "   Proportion of informative cases in our experiment: " `p_critical_expt'
	di "   Proportion of informative cases in simulated data: " `p_critical_sim'
	di "   Informative cases were " `p_critical_expt'/`p_critical_sim' " times more common in our experiment."

	di _n(2) "How large of a sample would we have needed to get an equal number of critical tests?"
	di "   Total number of informative cases in our sample    : 261+320 = 581"
	di "   Average number of informative cases in simulations : " `n_critical_sim'
	di "   # of interviewees needed to get 581 critical tests : " _N * 581 / `n_critical_sim'
end
