###############################################################################
## INSTRUCTIONS
##
## The sections below are meant to run in R in reverse order.
##
## First, load the functions below. These are used to construct the confidence
## intervals in Figure 2 in the paper.
##
## Second, run the section that produces the confidence interval files. Note
## that R hangs if you use too much of the experimental data. (The reasons
## are a bit mysterious to me, though playing around with it a bit shows that
## using smaller but still large fractions of the data doesn't change things
## very much.) This step produces the files ci75.csv and ci95.csv that I use
## in the plotting stage.
##
## Finally, plot the data. Be sure you're in the correct working directory,
## load the files, and plot away. I haven't included axes labels and tics
## and such; feel free to add these.
##
## A couple notes. First, you may encounter situations where points fall 
## outside the simplex boundary indicated by the black lines. This means,
## based on simulated choices and Tomz and Van Houweling's estimation
## method, that one of the predicted frequencies is negative. This came up
## in the T&VH experimental data as well (when they did the bootstrap
## sampling to get confidence intervals) and their solution was to drop
## these estimates. I do the same. Usually the problem is that
## the mean number of categories in a simulation is less than about two,
## in which case there are a lot of purely random choices and the
## predictions become meaningless.
##
## Second, there are times when a prediction falls inside the simplex
## but well away from the path shown in Figure 2. Usually the problem is
## the same: too few categories on average. Check to see that the mean
## number of categories is a bit above two, and you should be able to
## reproduce Figure 2.

###############################################################################
## Plot the data.
###############################################################################

## set working directory to the Data/ directory in QPackage. Modifying the line below as needed.
setwd("Data/")

ci95 <- read.csv("ci95.csv")
ci75 <- read.csv("ci75.csv")
b <- read.csv("voteData5-bte.csv")
g <- read.csv("voteData5-gte.csv")

b <- subset(b, b$meanNumCat > 2 & b$disc >= 0 & b$dir >= 0 & b$prox >= 0)
g <- subset(g, g$meanNumCat > 2 & g$disc >= 0 & g$dir >= 0 & g$prox >= 0)

plot(c(0,1,0.5,0),c(0,0,sqrt(3)/2,0), type = 'l')
lines(ci95$x, ci95$y, col = "red")
lines(ci75$x, ci75$y, col = "red", lty=2)
points(c(0.276+0.577/2),c(sqrt(3)*0.577/2), col = "red", xlim=c(0.15,0.7), ylim=c(0.4,sqrt(3)/2))
points(b$disc+b$prox/2,sqrt(3)*b$prox/2, col = "blue")
points(g$disc+g$prox/2,sqrt(3)*g$prox/2, col = "green")


###############################################################################
## 95% and 75% confidence intervals for all data
###############################################################################

## RUN THIS PART ONLY ONCE!!! THERE'S NO NEED TO RUN IT EVERY TIME YOU REPLOT!!
## except maybe the last bit to find how far in the predicted values go.
## DATA GENERATED BY TV'S ANALYSIS.DO & EST_RATES_BOOT.ADO
library(foreign)
bb <- read.dta("../TomzVanHouweling_2008/savebootsAll.dta")
mm <- matrix(c(bb$pidisc+bb$piprox/2,sqrt(3)*bb$piprox/2), ncol=2, nrow=length(bb[,1]))
NN <- length(mm[,1])
while(length(mm[,1]) > 0.95*NN) {
  o <- order.ccw(mm)
  mm <- mm[-o,]
}
o <- order.ccw(mm)
o95 <- c(o,o[1])
lines(mm[o95,1],mm[o95,2], col = "red")
write.table(mm[o95,], file="ci95.csv", quote = FALSE, sep = ",",
            row.names=FALSE, col.names = c("x", "y"))

## for some reason this gets stuck if I use all the data (at 85.24%, oddly enough)
## so use only part of it.
mm <- matrix(c(bb$pidisc+bb$piprox/2,sqrt(3)*bb$piprox/2), ncol=2, nrow=length(bb[,1]))
mm <- mm[1:4500,]
NN <- length(mm[,1])
while(length(mm[,1]) > 0.75*NN) {
  o <- order.ccw(mm)
  mm <- mm[-o,]
}
o <- order.ccw(mm)
o75 <- c(o,o[1])
lines(mm[o75,1],mm[o75,2], col = "red")
write.table(mm[o75,], file="ci75.csv", quote = FALSE, sep = ",",
            row.names=FALSE, col.names = c("x", "y"))


###############################################################################
## FUNCTIONS: some have weird names. they're from Numerical Recipes.         ##
###############################################################################
## find the convex hull of a set of points
cnvxhull <- function(m) {
  or <- order(m[,2],m[,1])
  m <- m[or,]
  s <- 1
  o <- 1
  ch <- c()
  n <- 0
  while(s != n) { ## can't use o here or else the loop won't start
    ch <- c(ch,o)
    ## the following method **ASSUMES** the order
    ## determined above, i.e., order(m[,2],m[,1])
    pm <- m
    p0 <- m[o,]
    pm <- pm - matrix(rep(p0,dim(pm)[1]), ncol=dim(pm)[2], byrow=TRUE)
    check <- c()
    for (i in 1:dim(pm)[1]) {
      if (pm[i,1] != 0 | pm[i,2] != 0) { check <- c(check,i) }
    }
    argmin <- check[1]
    for (i in check[2:length(check)]) {
      vecmin <- pm[argmin,]
      vecnew <- pm[i,]
      test <- vecmin[1]*vecnew[2]-vecmin[2]*vecnew[1]
      if (abs(test) < 1e-12) { test = 0 }
      if (test < 0) { argmin <- i }
      if (test == 0 & abs(vecmin[1]) < abs(vecnew[1])) { argmin <- i }
    }
    o <- n <- argmin
  }
  return(or[ch])
}

## this is just the same as cnvxhull, except that it includes all colinear
## points on a line segment
order.ccw <- function(m) {
  or <- order(m[,2],m[,1])
  m <- m[or,]
  s <- 1
  o <- 1
  ch <- c()
  n <- 0
  while(s != n) { ## can't use o here or else the loop won't start
    ch <- c(ch,o)
    ## the following method **ASSUMES** the order
    ## determined above, i.e., order(m[,2],m[,1])
    pm <- m
    p0 <- m[o,]
    pm <- pm - matrix(rep(p0,dim(pm)[1]), ncol=dim(pm)[2], byrow=TRUE)
    check <- c()
    for (i in 1:dim(pm)[1]) {
      if (pm[i,1] != 0 | pm[i,2] != 0) { check <- c(check,i) }
    }
    argmin <- check[1]
    for (i in check[2:length(check)]) {
      vecmin <- pm[argmin,]
      vecnew <- pm[i,]
      test <- vecmin[1]*vecnew[2]-vecmin[2]*vecnew[1]
      if (abs(test) < 1e-12) { test = 0 }
      if (test < 0) { argmin <- i }
      if (test == 0 & abs(vecmin[1]) > abs(vecnew[1])) { argmin <- i }
    }
    o <- n <- argmin
  }
  return(or[ch])
}

## checks whether two line segments intersect
## principle: if p1 and p2 are on opposite sides of q1-q2 and vice-versa,
## then p1-p2 and q1-q2 intersect
intersect <- function(p1,p2,q1,q2) {
  ## first find perpendicular vectors.
  p <- p2-p1
  qp1 <- q1-p1
  qp2 <- q2-p1
  pp <- c(-p[2],p[1])
  q <- q2-q1
  pq1 <- p1-q1
  pq2 <- p2-q1
  qq <- c(-q[2],q[1])
  x <- y <- NA
  if( qp1%*%pp * qp2%*%pp <= 0 & pq1%*%qq * pq2%*%qq <= 0 ) {
      x <- q1[1]+((p1[2]*(p2[1]-q1[1])+p2[2]*q1[1]-p2[1]*q1[2]+
              p1[1]*(-p2[2]+q1[2]))*(q1[1]-q2[1]))/
                ((p1[2]-p2[2])*(q1[1]-q2[1])-(p1[1]-p2[1])*(q1[2]-q2[2]))
      y <- q1[2]+((p1[2]*(p2[1]-q1[1])+p2[2]*q1[1]-p2[1]*q1[2]+
              p1[1]*(-p2[2]+q1[2]))*(q1[2]-q2[2]))/
                ((p1[2]-p2[2])*(q1[1]-q2[1])-(p1[1]-p2[1])*(q1[2]-q2[2]))
  }
  return(c(x,y))
}

plybndy.intersect <- function(m1,m2) {
  im <- matrix(ncol=dim(m1)[2])
  pm1 <- rbind(m1,m1[1,])
  pm2 <- rbind(m2,m2[1,])
  for ( i in 1:dim(m1)[1] ) {
    for ( j in 1:dim(m2)[1] ) {
      pn <- matrix(intersect(pm1[i,],pm1[i+1,],pm2[j,],pm2[j+1,]),ncol=dim(m1)[2])
      if ( !is.na(pn[1]) ) {
        if ( is.na(im[1,1]) ) { 
          im[1,] <- pn
        } else {
          im <- rbind(im,pn)
        }
      }
    }
  }
  return(im)
}

## checks whether a point p is inside a convex polygon defined by matrix m
## assumes that points in m are ordered ccw
inside <- function(p,m) {
  test <- TRUE
  pm <- rbind(m,m[1,])
  for ( i in 1:dim(m)[1] ) {
    if ( right.turn(pm[i,],pm[i+1,],p) ) {
      test <- FALSE
    }
    ## cat(i,test,right.turn(pm[i,],pm[i+1,],p), pm[i,], pm[i+1,],"\n")
  }
  return(test)
}

left.turn <- function(a,b,c) {
  bma <- c(b[1]-a[1],b[2]-a[2])
  cmb <- c(c[1]-b[1],c[2]-b[2])
  test <- bma[1]*cmb[2]-bma[2]*cmb[1] > 0
  return(test)
}

right.turn <- function(a,b,c) {
  bma <- c(b[1]-a[1],b[2]-a[2])
  cmb <- c(c[1]-b[1],c[2]-b[2])
  test <- bma[1]*cmb[2]-bma[2]*cmb[1] < 0
  return(test)
}

polygon.intersect <- function(m1,m2) {
  im <- plybndy.intersect(m1,m2)
  for (i in 1:dim(m1)[1]) { if ( inside(m1[i,],m2) ) { im <- rbind(im,m1[i,]) } }
  for (i in 1:dim(m2)[1]) { if ( inside(m2[i,],m1) ) { im <- rbind(im,m2[i,]) } }
  o <- order.ccw(im)
  return(im[o,])
}
