#!/usr/bin/perl -w
use strict;
use IO::Handle;
#-----------------------------------------------------------------------------
# polSustain: a version of the SUSTAIN algorithm for studying political 
# preferences. 
#                                                                             
# see Love, Medin, and Gureckis, Psych. Rev. 111(2), 309-332, 2004.           
#-----------------------------------------------------------------------------

#-----------------------------------------------------------------------------
# INSTRUCTIONS FOR REPLICATING FIG. 2 IN "CATEGORIZATION-BASED SPATIAL VOTING
# 
# First, run polSustainPref5.pl -bte 100 and polSustainPref5.pl -gte 100 50. 
# Then, run the commands in plotData.r in R to generate a plot which will look
# like Figure 2. Note that I use default seeds for all calls to rand(); the
# randomization is present only to ensure that the parameter space is well
# covered and, in the case of candidate sampling (line 263), that the policy
# space is well covered. One can verify that this is the case by examining the
# output file.
#-----------------------------------------------------------------------------

#-----------------------------------------------------------------------------
# OVERVIEW: One run through the process goes like this.  Present a candidate
# (a point in [0,1]) to a simulated voter. Calculate the activations for each 
# category for this candidate (using getActivations). Determine which, if any,
# category the object belongs in (using getObjectInfo()).  Update the 
# categories with updateCategory(), which also updates the attentional tuning 
# parameters lambda. Repeat.
#-----------------------------------------------------------------------------
# This version implements the Tomz-VanHouweling
# tests for proximity-vs.-directional preferences.
#-----------------------------------------------------------------------------
# NOTE: this version is intended to use 1D spaces only!
#-----------------------------------------------------------------------------

#-----------------------------------------------------------------------------
# CONSTANTS                                                                 
#-----------------------------------------------------------------------------

# IRRATIONALS
use constant E  => 2.718281828;
use constant PI => 3.141592654;

# LOGICALS
use constant TRUE  => 1;
use constant FALSE => 0;

# SYSTEM CODES
use constant MISSING => -999;        # missing value code

#-----------------------------------------------------------------------------
# PARAMETER DEFAULTS
#-----------------------------------------------------------------------------

my $RANGE = 1;
my $N = 0.5;

my $DIST_SAMPLES = 250;
my $NUM_SAMPLES = 100000;

my $THRESHOLD = 0.5; # default threshold activation for new categories
my $ETA       = 0.2; # default learning rate parameter

#-----------------------------------------------------------------------------
# INITIALIZATION
#-----------------------------------------------------------------------------

# seed the random number generator for replication purposes.
srand(54); # 42 in base 10

#-----------------------------------------------------------------------------
# open the ada file and create the candidate distribution
#-----------------------------------------------------------------------------
open ADA, "< ADA/2000.csv"; # from Sarah Anderson's adjusted ADA scores
my @ada;
while(<ADA>) {
    chomp($_);
    $_ =~ s/\s+//g;
    my @fields = split ",", $_;
    push @ada, $fields[5] if $fields[0] <= 2006;
}
close ADA;
# normalize
my $adaMin = &min( \@ada );
my $adaMax = &max( \@ada );
my $diff = $adaMax-$adaMin;
for ( 0..$#ada ) { $ada[$_] = ($ada[$_]-$adaMin)/$diff; }

#-----------------------------------------------------------------------------
# open the tvh data files
#-----------------------------------------------------------------------------
open TVH05, "< TomzVanHouweling_2008/tvh2005.csv";
<TVH05>; #first line is a header
my @v05; my @q05;
while (<TVH05>) {
    chomp($_);
    $_ =~ s/\s+//g;
    my @fields = split ",", $_;
    push @v05, $fields[0];
    push @q05, $fields[4];
}

open TVH06, "< TomzVanHouweling_2008/tvh2006.csv";
<TVH06>; #first line is a header
my @v06; my @q06;
while (<TVH06>) {
    chomp($_);
    $_ =~ s/\s+//g;
    my @fields = split ",", $_;
    push @v06, $fields[0];
    push @q06, $fields[1];
}

# compute joint distributions over ideal points and statuses quo
# we'll use vDist to do rejection sampling on 
# the generated ideal points, and given v we'll sample from qDist to 
# generate statii quo.
my @vDist05 = (0,0,0,0,0,0,0,0,0,0,0);
my @qDist05 = ();
for ( 0..10 ) { push @qDist05, [ 0,0,0,0,0,0,0,0,0,0,0 ]; }
for my $i ( 0..$#v05 ) {
    $vDist05[$v05[$i]+5]++;
    $qDist05[$v05[$i]+5][$q05[$i]+5]++;
}
for my $i ( 0..10 ) {
    for my $j ( 0..10 ) { 
 	$qDist05[$i][$j] = $qDist05[$i][$j]/$vDist05[$i] 
	    unless $vDist05[$i] == 0; 
    }
    $vDist05[$i] = $vDist05[$i]/($#v05 + 1) unless $#v05+1 == 0;
}

my @vDist06 = (0,0,0,0,0,0,0,0,0,0,0);
my @qDist06 = ();
for ( 0..10 ) { push @qDist06, [ 0,0,0,0,0,0,0,0,0,0,0 ]; }
for my $i ( 0..$#v06 ) {
    $vDist06[$v06[$i]+5]++;
    $qDist06[$v06[$i]+5][$q06[$i]+5]++;
}
for my $i ( 0..10 ) {
    for my $j ( 0..10 ) { 
 	$qDist06[$i][$j] = $qDist06[$i][$j]/$vDist06[$i] 
	    unless $vDist06[$i] == 0; 
    }
    $vDist06[$i] = $vDist06[$i]/($#v06 + 1) unless $#v06+1 == 0;
}

#-----------------------------------------------------------------------------
# open the tvh codebook for 2006 candidate assignments
# this is the file that determines the set of candidates a voter with a given
# ideal point could see. the voter is equally likely to see any such pair.
#-----------------------------------------------------------------------------
open CODES, "< TomzVanHouweling_2008/candLookup2006.csv";
<CODES>; # header
my @candRules = ( [], [], [], [], [], [], [], [], [], [], [] );
while (<CODES>) {
    chomp($_);
    $_ =~ s/\s+//g;
    my @fields = split ",", $_;
    my $ideal = $fields[0];
    my $left  = ($fields[1] + 5)/10;
    my $right = ($fields[2] + 5)/10;
    push @{$candRules[$ideal+5]}, [$left, $right];
}

#-----------------------------------------------------------------------------
# open the main output file
#-----------------------------------------------------------------------------
my $outputFile = "voteData5".$ARGV[0].".csv";
open OUTPUT, "> Data/$outputFile";
print OUTPUT "par1, par2, meanNumObj, meanNumCat, ";
print OUTPUT "prox, disc, dir, pi1, pi2, pi6, s1, s2, s6, ";
print OUTPUT "tau, eta, Nsamp\n";

#-----------------------------------------------------------------------------
# MAIN LOOP
#-----------------------------------------------------------------------------

$| = 1;
OUTPUT->autoflush(1);

for my $dist ( 1..$DIST_SAMPLES ) {
    
    # some initialization stuff
    my $meanNumCat = 0;
    my $meanNumObj = 0;
    my $scenario1 = 0;
    my $scenario2 = 0;
    my $scenario6 = 0;
    my $pi1 = 0; my $pi2 = 0; my $pi6 = 0;
    my $fdir = 0; my $fdisc = 0; my $fprox = 0;
    my $findiff = 0;

    my ($modeNumObj, $varNumObj, $maxNumObj, $numObjProb, 
	$minNumObj, $delNumObj, $total);

    if ( $ARGV[0] =~ /-g/ ) { # GAUSSIAN
	$modeNumObj = $ARGV[1] * rand();
	$varNumObj =  $ARGV[2] * rand();
    } elsif ( $ARGV[0] =~ /-b/ ) { # BINOMINAL
 	my $tmp = 0;
 	while ( $tmp < 2 ) {
	    $maxNumObj = int( $ARGV[1] * rand()); 
	    $numObjProb = rand();
	    $tmp = $maxNumObj * $numObjProb;
	}
    } elsif ( $ARGV[0] =~ /-u/ ) { # UNIFORM
	$minNumObj = 1;
	$delNumObj = $ARGV[1] * rand();
    } elsif ( $ARGV[0] =~ /-f/ ) { # FIXED
	$total = 1 + int($ARGV[1] * rand());
    } elsif ( $ARGV[0] =~ /-a/ ) { # ALL THE SAME
	$total = $ARGV[1];
    } else {
	die 'Invalid option';
    }

    # for parameter robustness checks
    if ( $ARGV[0] =~ /-\w*t/ ) { # sample TAU
	$THRESHOLD = 0.8 * rand();
    }
    if ( $ARGV[0] =~ /-\w*e/ ) { # sample ETA
	$ETA = rand();
    }

    my $actualSamples = 0;
    my $samples2005 = 0;

    for my $sample ( 1..$NUM_SAMPLES ) {
#	print " Working on dist $dist, sample $sample    \r";
	
	# some initialization stuff
	my @catArray = ();  # reset category array
	my $lambda = 1/$RANGE; # see LMG following Equation 13

	# compute the number of objects for this sample
	my $numObj = 0;
	if ( $ARGV[0] =~ /-g/ ) { # GAUSSIAN
	    $numObj = int(&gaussRand($modeNumObj, $varNumObj));
	    while ( $numObj < 1 ) {
		$numObj = int(&gaussRand($modeNumObj, $varNumObj));
	    }
	} elsif ( $ARGV[0] =~ /-b/ ) { # BINOMINAL
	    while ( $numObj < 2 ) {
		for ( 0..$maxNumObj ) { $numObj++ if rand() < $numObjProb; }
	    }
	} elsif ( $ARGV[0] =~ /-u/ ) { # UNIFORM
	    $numObj = $minNumObj + int($delNumObj * rand());
	} elsif ( $ARGV[0] =~ /-f/ ) { # FIXED
	    $numObj = $total;
	} elsif ( $ARGV[0] =~ /-a/ ) { # ALL THE SAME
	    $numObj = $total;
	}

	#######################################################################
	#######################################################################
	# OBSERVATION AND CATEGORIZATION STUFF
	#######################################################################
	#######################################################################

	#----------------------------------------------------------------------
	# start the categorization algorithm
	for my $objNum ( 0..$numObj - 1 ) {

	    # get an object. . . 
	    my $object = $ada[int($#ada * rand())];
	    
	    # compute the category activations. See LMG Eq. 5.
	    my @activations = &getActivations1D( \@catArray, $object, $lambda);
	    
	    # $newCat is a flag set to one if a new category is to be
	    # created; $winCat indicates the most activated category;
	    # $winAct is the activation of this category.
	    my ( $newCat, $winCat, $winAct ) 
		= &getObjectInfo( \@activations, $THRESHOLD );
	    
	    # update the categories accordingly.  See LMG Eqs. 12 and 13. 
	    if ( $newCat == TRUE ) {
		$winCat = $#catArray + 1;
		$catArray[ $winCat ] = $object ;
		$newCat = FALSE;
	    }
	    # NOTE: because $newCat = FALSE by the time you get here,
	    # the attn vector lambda WILL be updated next.
	    my $dist = $object - $catArray[$winCat];
	    $catArray[$winCat] += $ETA * $dist;
	    $lambda += $ETA * exp( - $lambda * abs( $dist ) ) 
		* ( 1 - $lambda * abs( $dist ) );
	    
	} # end of object loop
	#----------------------------------------------------------------------

	if ( $#catArray < 0 ) {
	    print "ERROR: Voter observed no candidates.\n";
	    die;
	}
	
	@catArray = sort { $a <=> $b } @catArray;

	#######################################################################
	#######################################################################
	# CHOICE STUFF
	#######################################################################
	#######################################################################

	# First choose whether this will be a tvh 2005 or 2006 scenario
	my $qYear = 2005; $qYear = 2006 if rand() < 0.4486133;
	$samples2005++ if $qYear == 2005;

	#----------------------------------------------------------------------
	# CHOOSE AN IDEAL POINT
        # first choose an ideal category at random
	my $idealCat = -1;
	while ( $idealCat < 0 || $idealCat > $#catArray ) {
	    $idealCat = int( ($#catArray+1) * rand() ); 
	}
	my $ideal = $catArray[$idealCat];
	my $trueIdeal = $ideal;
	# now round to nearest tenth
	$ideal = int(10 * $ideal + 0.5)/10;
 	# rejection sampling!
 	next if ($qYear == 2005 
 		 && (rand() > $vDist05[10*$ideal] || $ideal == 0 
 		     || $ideal == 1 || $ideal == 0.5));
 	next if ($qYear == 2006 
 		 && (rand() > $vDist06[10*$ideal] || $ideal == 0 
 		     || $ideal == 0.1 || $ideal == 1 
 		     || $ideal == 0.9|| $ideal == 0.5));

# 	next if ($ideal==0 || $ideal == 1 || $ideal == 0.5);
# 	next if ($qYear == 2006 && ($ideal == 0.1 || $ideal == 0.9));

	#----------------------------------------------------------------------
	# update averages; do the division later
	$meanNumObj += $numObj;
	$meanNumCat += $#catArray + 1;
	$actualSamples++;
	
	#----------------------------------------------------------------------
	# CHOOSE A STATUS QUO
	my $Q;
 	my $tmpRand = rand();
 	my $tmpProb = 0;
 	for ( 0..10 ) {
 	    my $flag = 0;
 	    $tmpProb += $qDist05[10*$ideal][$_] if $qYear == 2005;
 	    $tmpProb += $qDist06[10*$ideal][$_] if $qYear == 2006;
 	    if ( $tmpRand < $tmpProb ) {
 		$flag = 1;
 		$Q = $_/10;
 	    }
 	    last if $flag == 1;
 	}
	
	#----------------------------------------------------------------------
	# CHOOSE CANDIDATES USING TVH ALGORITHM

	my ($cand1, $cand2);
	if ( $qYear == 2005 ) {
	    my $tmpRand = rand();
	    if ( $ideal == 0.1 || $ideal == 0.9) { 
		($cand1, $cand2) = (0,0.1);
	    } elsif ( $ideal == 0.2 || $ideal == 0.8) { 
		($cand1, $cand2) = (0.1,0.2); 
	    } elsif ( $ideal == 0.3 || $ideal == 0.7 ) {
		if      ( $tmpRand < 1/4 ) { ($cand1, $cand2) = (0.2,0.3);
		} elsif ( $tmpRand < 1/2 ) { ($cand1, $cand2) = (0.1,0.2);
		} elsif ( $tmpRand < 3/4 ) { ($cand1, $cand2) = (0.1,0.3); 
		} else                     { ($cand1, $cand2) = (0.1,0.4); }
	    } elsif ( $ideal == 0.4 || $ideal == 0.6 ) {
		if      ( $tmpRand < 1/6 ) { ($cand1, $cand2) = (0.4,0.3);
		} elsif ( $tmpRand < 2/6 ) { ($cand1, $cand2) = (0.4,0.2);
		} elsif ( $tmpRand < 3/6 ) { ($cand1, $cand2) = (0.4,0.1);
		} elsif ( $tmpRand < 4/6 ) { ($cand1, $cand2) = (0.3,0.2);
		} elsif ( $tmpRand < 5/6 ) { ($cand1, $cand2) = (0.3,0.1);
		} else                     { ($cand1, $cand2) = (0.2,0.1); }
	    } else {
		die "Invalid ideal point: $ideal";
	    }
	    # reflect candidates if $ideal > 0.5
	    if ( $ideal > 0.5 ) {
		$cand1 = 1 - $cand1;
		$cand2 = 1 - $cand2;
	    }
	} elsif ( $qYear == 2006 ) {
	    	my $ri = -1;
		while ( $ri < 0 || $ri > $#{$candRules[10 * $ideal]} ) {
		    $ri = int(rand() * $#{$candRules[10 * $ideal]}); 
		}
		($cand1, $cand2) = @{$candRules[10 * $ideal][$ri]};
	}

	# swap candidates with probability 1/2
	if ( rand() < 0.5 ) { 
	    my $tmp = $cand1; $cand1 = $cand2; $cand2 = $tmp; }
	
	my $candMidpoint = ($cand1 + $cand2)/2;
	
#	print "Candidates: $cand1, $cand2\n";
	
	#----------------------------------------------------------------------
	# FIND CANDIDATE CATEGORIES
	my @act1 = &getActivations1D( \@catArray, $cand1, $lambda );
	my $k1 = $catArray[ &argmax( \@act1 ) ];

	my @act2 = &getActivations1D( \@catArray, $cand2, $lambda );
	my $k2 = $catArray[ &argmax( \@act2 ) ];

	#----------------------------------------------------------------------
	# COMPUTE CHOICES FOR EACH MODEL (except disc, which we can't compute)
	my ( $prox, $dir, $choice ) = ( 0,0,0 ); 

	$ideal = $trueIdeal;

	# categorization model
 	$choice = 1 if abs( $ideal - $k1 ) < abs( $ideal - $k2 );
 	$choice = 2 if abs( $ideal - $k1 ) > abs( $ideal - $k2 );

	# indifference
 	if ( $k1 == $k2 ) {
	    $findiff++;
 	    $choice = 1;
 	    $choice = 2 if rand() > 1/2;
 	}
	
	# prox
	$prox = 1 if abs( $ideal - $cand1 ) < abs( $ideal - $cand2 );
	$prox = 2 if abs( $ideal - $cand1 ) > abs( $ideal - $cand2 );

	# dirRM
	my $uDir1 = ($cand1 - $N) * ($ideal - $N);
	my $uDir2 = ($cand2 - $N) * ($ideal - $N);
	$dir = 1 if $uDir1 > $uDir2;
	$dir = 2 if $uDir1 < $uDir2;

	next if ( $choice == 0 || $prox == 0 || $dir == 0 );
	
	#----------------------------------------------------------------------
	# identify the scenarios and determine the pi_i
	# SCENARIO I
 	# isolates directional voting
	if ( ( $N < $ideal && $ideal <= $Q && $ideal < $candMidpoint )
	     || ( $N > $ideal && $ideal >= $Q && $ideal > $candMidpoint ) ) {
	    $scenario1++;
	    $pi1++ if $choice == $dir;
	}

	# SCENARIO II - error rate
	# all standard models agree
	if ( (($N < $ideal) && ($Q <= $ideal) && ($ideal > $candMidpoint))
	     || (($N > $ideal) && ($Q >= $ideal) && ($ideal < $candMidpoint))) {
	    $scenario2++;
	    $pi2++ if $choice != $prox;
	}

	# SCENARIO VI
	# proximity is different; NB we're looking for $choice != $prox
	# need to compute alpha*
	my $alphaStar = 1;
	$alphaStar = ($ideal-$candMidpoint)/($Q-$candMidpoint) 
	    unless $Q == $candMidpoint;
	if (($N<$ideal && $Q<$ideal && $ideal<$candMidpoint && $alphaStar<0.1)
	    ||($N>$ideal && $Q>$ideal && $ideal>$candMidpoint && $alphaStar<0.1)) {
	    $scenario6++;
	    $pi6++ if $choice != $prox;
	}

    } # end of the sample loop

    #-------------------------------------------------------------------------
    # compute fractions
    print "Warning 1\n" if $scenario1 == 0;
    print "Warning 2\n" if $scenario2 == 0;
    print "Warning 6\n" if $scenario6 == 0;
    $pi1 = $pi1/$scenario1 unless $scenario1 == 0;
    $pi2 = $pi2/$scenario2 unless $scenario2 == 0;
    next if $pi2 == 0.50;
    $pi6 = $pi6/$scenario6 unless $scenario6 == 0;
    $fdir = ($pi1 - $pi2)/(1 - 2 * $pi2);
    print "Warning: dir < 0                                   \n"
	if $fdir < 0;
    $fdisc = ($pi6 - $pi1)/(1 - 2 * $pi2);
    $fprox = 1 - $fdir - $fdisc;

    # update averages
    $meanNumObj /= $actualSamples; 
    $meanNumCat /= $actualSamples; 
    $findiff /= $actualSamples;
#    print "$findiff, $meanNumCat                       \n"; 

    #-------------------------------------------------------------------------
    # output
    if ( $ARGV[0] =~ /-g/ ) { # GAUSSIAN
	print OUTPUT "$modeNumObj, $varNumObj, ";
    } elsif ( $ARGV[0] =~ /-b/ ) { # BINOMINAL
	print OUTPUT "$maxNumObj, $numObjProb, ";
    } elsif ( $ARGV[0] =~ /-u/ ) { # UNIFORM
	print OUTPUT "$minNumObj, $delNumObj, ";
    } elsif ( $ARGV[0] =~ /-f/ ) { # FIXED
	print OUTPUT "$total, 0, ";
    } elsif ( $ARGV[0] =~ /-a/ ) { # ALL THE SAME
	print OUTPUT "$total, 0, ";
    }

    print OUTPUT "$meanNumObj, $meanNumCat, $fprox, $fdisc, $fdir, ";
    print OUTPUT "$pi1, $pi2, $pi6, $scenario1, $scenario2, $scenario6, ";
    print OUTPUT "$THRESHOLD, $ETA, $actualSamples\n";

} # end of main loop (over distributions) 

#print "Done.                                                       \n";
exit;

#-----------------------------------------------------------------------------
#                            /\/ SUBROUTINES /\/
#-----------------------------------------------------------------------------

#-----------------------------------------------------------------------------
# CATEGORIZATION FUNCTIONS                          
#-----------------------------------------------------------------------------

#-----------------------------------------------------------------------------
# getObjectInfo() calculates information that we need to update categories.
# It takes a reference to an array of activations and a threshold.  It returns
# an array, ( $makeNewCategory, $mostActiveCategory, $largestActivation ).
#
# $makeNewCategory is TRUE if no activation is over the threshold and FALSE
# otherwise.  $mostActiveCategory is the position of the category with the
# largest activation.  $largestActivation is, of course, the largest 
# activation. 
#
# Because the code is somewhat error-prone, there are error-checking functions
# included at the end of the subroutine.
#
# see Love, Medin, and Gureckis (2004), Equations 5 and 11.
#-----------------------------------------------------------------------------

sub getObjectInfo {
    my $actRef    = shift;
    my $threshold = shift;

    my $makeNewCategory = TRUE;
    my $mostActiveCategory = MISSING;
    my $largestActivation = 0; 
    my @testPositions = ();

    # note: makeNewCategory is FALSE if activation is over the threshold
    # also note: if the activation subroutine couldn't compute an 
    # activation, it takes value MISSING, which is negative and therefore
    # below $threshold.
    for ( 0..$#$actRef ) {
	if ( $$actRef[$_] >= $threshold ) {
	    $makeNewCategory = FALSE;
	    push @testPositions, $_;  # for error-checking, below
	}
	if ( $$actRef[$_] > $largestActivation ) {
	    $largestActivation = $$actRef[$_];
	    $mostActiveCategory = $_;
	}
    }

    # some error-checking and diagnostics
    if ( $makeNewCategory == FALSE && $mostActiveCategory == MISSING ) {
	foreach ( @$actRef ) {
	    print "$_\n";
	}
	print "Largest: $largestActivation\n";
	foreach ( @testPositions ) {
	    print "$_, $$actRef[$_]\n";
	}
	die;
    }

    if ( $makeNewCategory == FALSE && $largestActivation < $threshold ) {
	foreach ( @$actRef ) {
	    print "$_\n";
	}
	print "Largest: $largestActivation\n";
	foreach ( @testPositions ) {
	    print "$_, $$actRef[$_]\n";
	}
	die;
    }

    return ( $makeNewCategory, $mostActiveCategory, $largestActivation );
}

#-----------------------------------------------------------------------------
# getActivations() is a shell for getting the activations corresponding to
# each of the categories.  Takes the list of categories (an array of refs to
# category objects, themselves refs to arrays of numbers), a ref to an object,
# the attentional tuning parameters, and the attentional parameter r (see LMG 
# Equation 5), which defaults to one.  Returns an array of activations.
#-----------------------------------------------------------------------------

sub getActivations1D {
    my $catArrayRef = shift; # the array of category objects
    my $obj         = shift; # the presented object
    my $lam         = shift; # ref to a vector of tuning parameters

    my @activationArray = ();
    for ( 0..$#$catArrayRef ) {
	$activationArray[$_] = exp( -$lam * abs( $obj - $$catArrayRef[$_] ) );
    }

    return @activationArray;
}
    
#-----------------------------------------------------------------------------
# UTILITY FUNCTIONS
#-----------------------------------------------------------------------------

sub max {
    my $vecRef = shift;
    my @vec = @$vecRef;
    my $max = $vec[0];
    for ( 1..$#vec ) {
	$max = $vec[$_] if $vec[$_] > $max;
    }
    return $max;
}

sub min {
    my $vecRef = shift;
    my @vec = @$vecRef;
    my $min = $vec[0];
    for ( 1..$#vec ) {
	$min = $vec[$_] if $vec[$_] < $min;
    }
    return $min;
}

sub argmax {
    my $vecRef = shift;
    my @vec = @$vecRef;
    my $pos = 0;
    for ( 1..$#vec ) {
	$pos = $_ if $vec[$_] > $vec[$pos];
    }
    return $pos;
}


#-----------------------------------------------------------------------------
# subroutine gaussRand                                     
#                                                          
# gaussRand generates an normally distributed random variable using an 
# exponential random variable and rejection sampling.  Takes a mean and 
# variance parameter and returns the sampled value. Defaults to a standard    
# normal distribution.                                     
#                                                          
# See Ross, _Simulation_, Example 5f.                      
#-----------------------------------------------------------------------------

# A word about the algorithm: the idea is to generate an
# RV that is easy to generate and has the same (or similar)
# domain as the RV we're trying to generate. Then use this RV
# to construct a map of the cdf onto [0,1], and use a U(0,1)
# RV to accept or reject the original sampled RV.

sub gaussRand {
    my $mean = shift;
    my $variance = shift;
    $mean = 0 unless $mean;
    $variance =  1 unless $variance;

    while ( ) { # yikes! but it tests okay.
	my $expVar = &expRand();
	if ( rand() <= exp( -( ( $expVar - 1 )**2 )/2 ) ) {
	    my $sign = -1;
	    $sign = 1 unless ( rand() <= 0.5 );
	    return $mean + sqrt($variance) * $sign * $expVar;
	}
    }
}

#-----------------------------------------------------------------------------
# subroutine expRand                                       
#                                                        
# expRand generates an exponentially distributed random variable using the 
# inverse transform method.  Takes a mean (1/rate) parameter and returns the 
# sampled value. The rate defaults to 1.                                
#-----------------------------------------------------------------------------

sub expRand {
    my $mean = shift;
    $mean = 1 unless $mean;
    my $num = 0;
    while ( $num == 0 ) { $num = rand(); }
    return -$mean * log( $num );
}
